==> efax-gtk_1.0.1a.orig.tar.gz: efax-gtk-1.0.1a.orig/pipes.h <==
/* Copyright (C) 1999 - 2001 Chris Vine

This program is distributed under the General Public Licence, version 2.
For particulars of this and relevant disclaimers see the file
COPYRIGHT distributed with the source files.

*/

#ifndef PIPES_H
#define PIPES_H

#ifndef CONFIG_H
#define CONFIG_H
#include "config.h"
#endif

#include <unistd.h>

#include "prog_defs.h"

/*

This class provides a simplified front end to unix write() and read()
of fifo pipes.  The constructor of the class may be passed an
enumerator to indicate whether the pipe is to read in non-blocking
mode.

The write(char*) method taking a string must have a string less than
PIPE_BUF characters in length, or it will fail.  With the write(char*,
int) method, taking an array and int arguments, the value of int
(indicating the number of chars in the array to be written) should
usually be less than PIPE_BUF, as POSIX write() guarantees that if it
is less than PIPE_BUF, either all will be written or, if the pipe is
too full, none will be written.  This prevents data interleaving if a
number of independent writes to the fifo are made.  If the pipe is too
full, write will return immediately with 0 if the pipe has been set to
read in non-blocking mode.

The pipe is set to read in non-blocking mode if the constructor or
Pipe_fifo is passed the value Pipe_fifo::non_block.  If constructed
in this way, the pipe can also be set to write in non-block mode (eg 
to minimise the impact on another program running in a child process
to which the child process has exec()ed when monitoring its stdin
or stderr) by calling make_write_non_block().  However use this very
sparingly -- when set in this way write() will always try to write
something.  The atomic guarantees mentioned in the preceding paragraph
do not apply and data can be interleaved or lost.

PIPE_BUF is defined in limits.h, and is at least 512 bytes, and usually 4096 bytes,
but may be calculated from other files included by limits.h.

Where a pipe is used to communicate between parent and child after a
call to fork(), the pipe will normally be used unidirectionally unless
guards or semaphores are used to prevent a process reading data
intended for the other.  However, because each process inherits its
own duplicate of the file descriptors, this cannot be enforced without
closing the read or write file descriptor for the process for which
the reading or writing is to be prohibited.  This can be done by the
process concerned calling the methods Pipe_fifo::make_writeonly() or
Pipe_fifo::make_readonly() after the fork.  If an attempt is made to
read or write to a descriptor closed in this way, the
Pipe_fifo::read() or Pipe_fifo::write() method will ensure that no
read or write will take place, and instead a -1 will be returned.

The methods Pipe_fifo::connect_to_stdin(), Pipe_fifo::connect_to_stdout()
and Pipe_fifo::connect_to_stderr() are available to be used in the child
process before it exec()s another program so as to connect the pipe to
that programs stdin, stdout or stderr.  The same pipe may be used to
connect to both stdout and stderr.  Pipe_fifo::connect_to_stdin()
cannot be used by a process after that process has called
Pipe_fifo::make_writeonly(), and Pipe_fifo::connect_to_stdout() and
Pipe_fifo::connect_to_stderr() cannot be used after the process has
called Pipe_fifo::make_readonly().  If that is attempted the methods
will return -1; otherwise they will return 0.  Furthermore, they should
only be used after the process creating the pipe has forked.  If the
connection to stdin, stdout or stderr is to be made before the fork,
this must be done by hand using close(), dup(), Pipe_fifo::get_write_fd()/
Pipe_fifo::get_read_fd() and Pipe_fifo::make_read_only()/
Pipe_fifo::make_write_only().

If Pipe_fifo::connect_to_stdin() is called by a method,
Pipe_fifo::make_readonly() will also be called, and if
Pipe_fifo::connect_to_stdout() or Pipe_fifo::connect_to_stderr() are
called, Pipe_fifo::make_writeonly() will also be called.  This will 
isolate the use of the pipe by the child process to stdin, stdout or
stderr, as appropriate.

*/


class Pipe_fifo {
public:
    enum Fifo_mode{block, non_block};
private:
    int read_fd;
    int write_fd;
    Fifo_mode read_blocking_mode;
    Fifo_mode write_blocking_mode;
public:
    int open(Fifo_mode);
    void close(void);
    ssize_t read(char*, size_t);        // returns number of chars read (0 on empty non-blocking fifo)
    int read(void);                     // returns char (or -1 on empty non-blocking fifo)

    ssize_t write(const char*);         // if the write is non-blocking (see above) the string
                                        // must not be longer than PIPE_BUF (defined
                                        // in limits.h) and it returns -1 if it is - otherwise
                                        // it returns the number of chars written.  The string
				        // must be null terminated.  All the bets are off if
                                        // not only the read end but also the write
                                        // end have been set to non-block so far as data
                                        // interleaving or loss is concerned.

    ssize_t write(const char*, size_t); // returns the number of chars written
    int write(char item) {return write(&item, 1);} // returns 1 if char written, else 0
    void make_writeonly(void);
    void make_readonly(void);
    int make_write_non_block(void);
    int get_read_fd(void) const {return read_fd;}
    int get_write_fd(void) const {return write_fd;}
    int connect_to_stdin(void);
    int connect_to_stdout(void);
    int connect_to_stderr(void);
    Pipe_fifo(Fifo_mode);
    Pipe_fifo(void);
    ~Pipe_fifo(void) {close();}
};

#endif

==> efax-gtk_2.0.5a.orig.tar.gz: efax-gtk-2.0.5a.orig/pipes.h <==
/* Copyright (C) 1999 - 2001 Chris Vine

This program is distributed under the General Public Licence, version 2.
For particulars of this and relevant disclaimers see the file
COPYRIGHT distributed with the source files.

*/

#ifndef PIPES_H
#define PIPES_H

#include "prog_defs.h"

#include <unistd.h>

/*

This class provides a simplified front end to unix write() and read()
of fifo pipes.  The constructor of the class may be passed an
enumerator to indicate whether the pipe is to read in non-blocking
mode.

The write(char*) method taking a string must have a string less than
PIPE_BUF characters in length, or it will fail.  With the write(char*,
int) method, taking an array and int arguments, the value of int
(indicating the number of chars in the array to be written) should
usually be less than PIPE_BUF, as POSIX write() guarantees that if it
is less than PIPE_BUF, either all will be written or, if the pipe is
too full, none will be written.  This prevents data interleaving if a
number of independent writes to the fifo are made.  If the pipe is too
full, write will return immediately with 0 if the pipe has been set to
read in non-blocking mode.

The pipe is set to read in non-blocking mode if the constructor or
Pipe_fifo is passed the value Pipe_fifo::non_block.  If constructed
in this way, the pipe can also be set to write in non-block mode (eg 
to minimise the impact on another program running in a child process
to which the child process has exec()ed when monitoring its stdin
or stderr) by calling make_write_non_block().  However use this very
sparingly -- when set in this way write() will always try to write
something.  The atomic guarantees mentioned in the preceding paragraph
do not apply and data can be interleaved or lost.

PIPE_BUF is defined in limits.h, and is at least 512 bytes, and usually 4096 bytes,
but may be calculated from other files included by limits.h.

Where a pipe is used to communicate between parent and child after a
call to fork(), the pipe will normally be used unidirectionally unless
guards or semaphores are used to prevent a process reading data
intended for the other.  However, because each process inherits its
own duplicate of the file descriptors, this cannot be enforced without
closing the read or write file descriptor for the process for which
the reading or writing is to be prohibited.  This can be done by the
process concerned calling the methods Pipe_fifo::make_writeonly() or
Pipe_fifo::make_readonly() after the fork.  If an attempt is made to
read or write to a descriptor closed in this way, the
Pipe_fifo::read() or Pipe_fifo::write() method will ensure that no
read or write will take place, and instead a -1 will be returned.

The methods Pipe_fifo::connect_to_stdin(), Pipe_fifo::connect_to_stdout()
and Pipe_fifo::connect_to_stderr() are available to be used in the child
process before it exec()s another program so as to connect the pipe to
that program's stdin, stdout or stderr.  The same pipe may be used to
connect to both stdout and stderr.  Pipe_fifo::connect_to_stdin()
cannot be used by a process after that process has called
Pipe_fifo::make_writeonly(), and Pipe_fifo::connect_to_stdout() and
Pipe_fifo::connect_to_stderr() cannot be used after the process has
called Pipe_fifo::make_readonly().  If that is attempted the methods
will return -1; otherwise they will return 0.  Furthermore, they should
only be used after the process creating the pipe has forked.  If the
connection to stdin, stdout or stderr is to be made before the fork,
this must be done by hand using close(), dup(), Pipe_fifo::get_write_fd()/
Pipe_fifo::get_read_fd() and Pipe_fifo::make_read_only()/
Pipe_fifo::make_write_only().

If Pipe_fifo::connect_to_stdin() is called by a method,
Pipe_fifo::make_readonly() will also be called, and if
Pipe_fifo::connect_to_stdout() or Pipe_fifo::connect_to_stderr() are
called, Pipe_fifo::make_writeonly() will also be called.  This will 
isolate the use of the pipe by the child process to stdin, stdout or
stderr, as appropriate.

*/


class Pipe_fifo {
public:
    enum Fifo_mode{block, non_block};
private:
    int read_fd;
    int write_fd;
    Fifo_mode read_blocking_mode;
    Fifo_mode write_blocking_mode;
public:
    int open(Fifo_mode);
    void close(void);
    ssize_t read(char*, size_t);        // returns number of chars read (0 on empty non-blocking fifo)
    int read(void);                     // returns char (or -1 on empty non-blocking fifo)

    ssize_t write(const char*);         // if the write is non-blocking (see above) the string
                                        // must not be longer than PIPE_BUF (defined
                                        // in limits.h) and it returns -1 if it is - otherwise
                                        // it returns the number of chars written.  The string
				        // must be null terminated.  All the bets are off if
                                        // not only the read end but also the write
                                        // end have been set to non-block so far as data
                                        // interleaving or loss is concerned.

    ssize_t write(const char*, size_t); // returns the number of chars written
    int write(char item) {return write(&item, 1);} // returns 1 if char written, else 0
    void make_writeonly(void);
    void make_readonly(void);
    int make_write_non_block(void);
    int get_read_fd(void) const {return read_fd;}
    int get_write_fd(void) const {return write_fd;}
    int connect_to_stdin(void);
    int connect_to_stdout(void);
    int connect_to_stderr(void);
    Pipe_fifo(Fifo_mode);
    Pipe_fifo(void);
    ~Pipe_fifo(void) {close();}
};

#endif
